@@ -19,25 +19,29 @@
 #ifndef __COMMON_H__
 #define __COMMON_H__
 
+#include <stdio.h>
+#include <stdlib.h>
+
 #include <gtk/gtk.h>
 
 #include "db_conn.h"
+#include "config.h"
 
 #define APPNAME "GtkSQL"
-#define VERSION "0.2"
-#define MAIL    "bbrox@mygale.org"
-#define URL     "http://www.mygale.org/~bbrox/GtkSQL/"
+#define VERSION "0.3"
+#define MAIL    "bbrox@multimania.com"
+#define URL     "http://www.multimania.com/bbrox/GtkSQL/"
 
 extern DBConnection *conn;
 
-GtkNotebookPage *gtk_notebook_get_bookpage(GtkNotebook *notebook,
-					   gint position) ;
-GtkWidget *gtk_notebook_get_page(GtkNotebook *notebook,
-				 gint position) ;
-GtkWidget *gtk_notebook_get_tab_label(GtkNotebook *notebook,
-				      gint position) ;
-GtkWidget *gtk_notebook_get_menu_label(GtkNotebook *notebook,
-				       gint position) ;
+/* GtkNotebookPage *gtk_notebook_get_bookpage(GtkNotebook *notebook,
+   gint position) ;
+   GtkWidget *gtk_notebook_get_page(GtkNotebook *notebook,
+   gint position) ;
+   GtkWidget *gtk_notebook_get_tab_label(GtkNotebook *notebook,
+   gint position) ;
+   GtkWidget *gtk_notebook_get_menu_label(GtkNotebook *notebook,
+   gint position) ; */
 
 void UpdateButtonSensitivity() ;
 
