@@ -139,8 +139,7 @@ static void text_export_ok_callback(GtkWidget *widget, gpointer data) {
   /* Start exporting ... */
   result = GetQueryResult(cd->query);
   
-  if ((result != NULL) &&
-      (conn->DBget_query_status(conn, result) == DB_QUERY_RECORDS_OK)) {
+  if (conn->DBget_query_status(conn, result) == DB_QUERY_RECORDS_OK) {
     int nb_tuples = conn->DBget_record_number(conn, result);
     int nb_cols = conn->DBget_field_number(conn, result);
     int *col_length = (int *) g_malloc(sizeof(int) * nb_cols);
@@ -284,13 +283,12 @@ static void html_export_ok_callback(GtkWidget *widget, gpointer data)
 
   /* Gets all the info we need before exporting */
   result = GetQueryResult(cd->query);
-  if ((result == NULL) ||
-      (conn->DBget_query_status(conn, result) != DB_QUERY_RECORDS_OK))
+  if (conn->DBget_query_status(conn, result) != DB_QUERY_RECORDS_OK)
     return;
   nb_tuples = conn->DBget_record_number(conn, result);
   nb_cols = conn->DBget_field_number(conn, result);
 
-  basename = GTK_ENTRY(cd->rec_per_page)->text;
+  basename = gtk_entry_get_text(GTK_ENTRY(cd->rec_per_page));
   rec_per_page = atoi(basename); /* FIXME: check the return value */
   basename = NULL;
   links = GTK_TOGGLE_BUTTON(cd->links)->active;
@@ -395,8 +393,7 @@ static void ExportAsHtml(char *filename, char *name,
   rec_per_page = gtk_entry_new();
 
   result = GetQueryResult(query);
-  if ((result == NULL) ||
-      (conn->DBget_query_status(conn, result) != DB_QUERY_RECORDS_OK))
+  if (conn->DBget_query_status(conn, result) != DB_QUERY_RECORDS_OK)
     return;
   sprintf(buf, "%d", conn->DBget_record_number(conn, result));
   gtk_entry_set_text(GTK_ENTRY(rec_per_page), buf);
