@@ -16,6 +16,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <string.h>
+
 #include "common.h"
 
 #include "queries.h"
@@ -23,10 +25,14 @@
 #include "tables.h"
 #include "export.h"
 #include "status.h"
+#include "mytoolbar.h"
 
-#ifdef POSTGRESQL
+#ifdef USE_POSTGRESQL
 #include "postgres.h"
 #endif
+#ifdef USE_MYSQL
+#include "mysql.h"
+#endif
 
 /* Icons for the toolbar */
 #include "Pics/gtksql.xpm"
@@ -52,17 +58,30 @@
 #include "Pics/export-grey.xpm"
 #include "Pics/refresh-grey.xpm"
 
-#define NORM_KEY "norm"
-#define GREY_KEY "grey"
-
 static register_func register_DB[] = {
-#ifdef POSTGRESQL
+#ifdef USE_POSTGRESQL
   register_PostgreSQL,
-#endif POSTGRESQL
+#endif USE_POSTGRESQL
+#ifdef USE_MYSQL
+  register_MySQL,
+#endif USE_MYSQL
   NULL
 };
+static GtkWidget *window = NULL;
+static DBConnector **connector_table = NULL;
+
+/* Open dialog variables and constants */
+#define DIALOG_FIELDS 0
+#define DIALOG_LIST   1
+
 static GtkWidget *open_dialog = NULL;
-static GtkWidget *window;
+static GtkWidget *open_dialog_notebook;
+static GtkWidget *open_dialog_list;
+static int open_dialog_status;
+
+/* Defines for submenus type */
+#define NORM_BUTTON   0
+#define TOGGLE_BUTTON 1
 
 /* Global exported variables */
 DBConnection *conn = NULL;
@@ -73,6 +92,7 @@ static void disconnect (GtkWidget *widget, gpointer data) ;
 static void about (GtkWidget *widget, gpointer data) ;
 static void do_query (GtkWidget *widget, gpointer data) ;
 static void refresh_tables (GtkWidget *widget, gpointer data) ;
+static void editor_display (GtkWidget *widget, gpointer data) ;
 static void add_query (GtkWidget *widget, gpointer data) ;
 static void delete_query (GtkWidget *widget, gpointer data) ;
 static void rename_query (GtkWidget *widget, gpointer data) ;
@@ -92,18 +112,9 @@ typedef struct _submenu {
   void (*callback)();
   gpointer data;
   GtkWidget **store_to;
+  int type;
 } SubMenu;
 
-typedef struct _toolbar {
-  char *label;
-  char **pixmap;
-  char **grey_pixmap;
-  char *tooltip;
-  void (*callback)();
-  gpointer data;
-  GtkWidget **store_to;
-} ToolBar;
-
 typedef struct _activator {
   GtkWidget **widget;
   int (*activator)();
@@ -133,6 +144,8 @@ GtkWidget *load_menu_item;
 GtkWidget *export_button;
 GtkWidget *save_button;
 GtkWidget *load_button;
+GtkWidget *refresh_tables_menu_item;
+GtkWidget *editor_menu_item;
 
 Activator SensWidget[] = {
   { &connect_button, ConnectionBad },
@@ -154,6 +167,8 @@ Activator SensWidget[] = {
   { &save_menu_item, QueryExists },
   { &load_menu_item, QueryExists },
   { &new_menu_item, ConnectionOK },
+  { &refresh_tables_menu_item, ConnectionOK },
+  /* { &editor_menu_item, ConnectionOK }, */
   { NULL, NULL}
 };
 
@@ -190,42 +205,48 @@ ToolBar toolbardef[] = {
   { "", NULL, NULL, NULL, NULL, NULL},
 
   { "Refresh", refresh_xpm, refresh_grey_xpm,
-    "Refresh the table display", refresh_tables,  NULL, &refresh_tables_button},
+    "Refresh the table display", refresh_tables, NULL, &refresh_tables_button},
 
   { NULL, NULL, NULL, NULL, NULL, NULL}
 };
 
 SubMenu DatabaseMenu[] = {
-  { "Connect...", connect_to_base, NULL, &connect_menu_item },
-  { "Disconnect", disconnect, NULL, &disconnect_menu_item },
-  { "", NULL, NULL, NULL },
-  { "Quit", destroy, NULL, NULL },
-  { NULL, NULL, NULL, NULL }
+  { "Connect...", connect_to_base, NULL, &connect_menu_item, NORM_BUTTON },
+  { "Disconnect", disconnect, NULL, &disconnect_menu_item, NORM_BUTTON },
+  { "", NULL, NULL, NULL, NORM_BUTTON },
+  { "Quit", destroy, NULL, NULL, NORM_BUTTON },
+  { NULL, NULL, NULL, NULL, NORM_BUTTON }
 };
 
 SubMenu QueryMenu[] = {
-  { "Send", do_query, NULL, &send_menu_item },
-  { "Rename", rename_query, NULL, &rename_menu_item },
-  { "Delete", delete_query, NULL, &delete_menu_item },
-  { "New", add_query, NULL, &new_menu_item },
-  { "", NULL, NULL, NULL },
-  { "Save", save_dialog, NULL, &save_menu_item },
-  { "Load", load_dialog, NULL, &load_menu_item },
-  { "", NULL, NULL, NULL },
-  { "Export...", export_dialog, NULL, &export_menu_item },
-  { NULL, NULL, NULL, NULL }
+  { "Send", do_query, NULL, &send_menu_item, NORM_BUTTON },
+  { "Rename", rename_query, NULL, &rename_menu_item, NORM_BUTTON },
+  { "Delete", delete_query, NULL, &delete_menu_item, NORM_BUTTON },
+  { "New", add_query, NULL, &new_menu_item, NORM_BUTTON },
+  { "", NULL, NULL, NULL, NORM_BUTTON },
+  { "Save", save_dialog, NULL, &save_menu_item, NORM_BUTTON },
+  { "Load", load_dialog, NULL, &load_menu_item, NORM_BUTTON },
+  { "", NULL, NULL, NULL, NORM_BUTTON },
+  { "Export...", export_dialog, NULL, &export_menu_item, NORM_BUTTON },
+  { NULL, NULL, NULL, NULL, NORM_BUTTON }
+};
+
+SubMenu TableMenu[] = {
+  { "Refresh", refresh_tables, NULL, &refresh_tables_menu_item, NORM_BUTTON },
+  /* { "Editor", editor_display, NULL, &editor_menu_item, TOGGLE_BUTTON }, */
+  { NULL, NULL, NULL, NULL, NORM_BUTTON }
 };
 
 SubMenu Help[] = {
-  { "About...", about, NULL, NULL },
-  { NULL, NULL, NULL, NULL }
+  { "About...", about, NULL, NULL, NORM_BUTTON },
+  { NULL, NULL, NULL, NULL, NORM_BUTTON }
 };
 
 SubMenu *Menu[] = {
-  DatabaseMenu, QueryMenu, Help, NULL
+  DatabaseMenu, QueryMenu, TableMenu, Help, NULL
 };
 char *MenuNames[] = {
-  "Database", "Query", "Help", NULL
+  "Database", "Query", "Table", "Help", NULL
 };
 
 static int ConnectionOK() {
@@ -235,6 +256,9 @@ static int ConnectionOK() {
     return 0;
 }
 static int ConnectionBad() {
+  if (open_dialog == NULL)
+    return 0;
+  
   if (conn == NULL)
     return 1;
   else
@@ -260,28 +284,10 @@ void UpdateButtonSensitivity() {
       
   i = 0;
   while (SensWidget[i].widget != NULL) {
-    void *v_pix;
+    int act = (SensWidget[i].activator)();
+    
+    gtk_widget_set_sensitive(*(SensWidget[i].widget), act);
     
-    if ((SensWidget[i].activator)()) {
-      gtk_widget_set_sensitive(*(SensWidget[i].widget), 1);
-      v_pix = gtk_object_get_data(GTK_OBJECT(*(SensWidget[i].widget)),
-				  NORM_KEY);
-    } else {
-      gtk_widget_set_sensitive(*(SensWidget[i].widget), 0);
-      v_pix = gtk_object_get_data(GTK_OBJECT(*(SensWidget[i].widget)),
-				  GREY_KEY);
-    }
-#if 0
-    /* There, we should have a nice greyed pixmap...
-       But I do not know how to do it :-) */
-    if (v_pix != NULL) {
-      GtkWidget *pix = GTK_WIDGET(v_pix);
-      struct _GtkToolbarChild *tbc =
-	(struct _GtkToolbarChild *) *(SensWidget[i].widget);
-      
-      tbc->icon = pix;
-    }
-#endif
     i++;
   }
 }
@@ -306,8 +312,13 @@ static void disconnect_callback(void *data) {
   TablesConnectionClosed();
   QueriesConnectionClosed();
 
+  /* Resets the queries counter */ 
+  InitQueries();
+  
   conn->DBdisconnect(conn);
   conn = NULL;
+
+  gtk_window_set_title(GTK_WINDOW(window), "GtkSQL");
   
   UpdateButtonSensitivity();
 }
@@ -361,8 +372,14 @@ static void about(GtkWidget *widget, gpointer data)
   GtkWidget *dialog;
   char buf[512];
 
-  sprintf(buf, "\n %s\n (c) 1998 Lionel ULMER \n Version %s \n %s \n",
+  sprintf(buf, "\n %s\n (c) 1998 Lionel ULMER \n Version %s \n %s \n\n",
 	  APPNAME, VERSION, URL);
+#ifdef USE_POSTGRESQL
+  strcat(buf, " PostgreSQL driver installed \n");
+#endif USE_POSTGRESQL
+#ifdef USE_MYSQL
+  strcat(buf, " MySQL driver installed \n");
+#endif USE_MYSQL
 
   dialog = CreateStandardDialog("About " APPNAME, buf,
 				0, gtksql_xpm,
@@ -391,8 +408,14 @@ static GtkWidget *CreateMenus() {
     while (cmenu[j].label != NULL) {
       if (strlen(cmenu[j].label) == 0)
 	item = gtk_menu_item_new();
-      else
-	item = gtk_menu_item_new_with_label(cmenu[j].label);
+      else {
+	if (cmenu[j].type == TOGGLE_BUTTON) {
+	  item = gtk_check_menu_item_new_with_label(cmenu[j].label);
+	  gtk_check_menu_item_set_show_toggle(GTK_CHECK_MENU_ITEM(item),
+					      TRUE);
+	} else
+	  item = gtk_menu_item_new_with_label(cmenu[j].label);
+      }
       
       gtk_menu_append(GTK_MENU(menu), item);
       if (cmenu[j].callback != NULL)
@@ -446,80 +469,71 @@ static void refresh_tables(GtkWidget *widget, gpointer data) {
   TablesConnectionClosed();
   TablesNewConnection();
 }
+static void editor_display(GtkWidget *widget, gpointer data) {
+  DisplayEditor();
+}
 
 static void CreateToolBar(GtkWidget *mainvbox) {
   GtkWidget *toolbar;
-  int i;
+  int tbc;
+  
+  if (conf.toolbar_conf == TOOLBAR_ICONS)
+    tbc = GTK_TOOLBAR_ICONS;
+  else if (conf.toolbar_conf == TOOLBAR_BOTH)
+    tbc = GTK_TOOLBAR_BOTH;
+  else if (conf.toolbar_conf == TOOLBAR_TEXT)
+    tbc = GTK_TOOLBAR_TEXT;
+  else {
+    fprintf(stderr, "Internal error : bad configuration value.\n");
+    tbc = GTK_TOOLBAR_BOTH;
+  }
+    
+  toolbar = CreateMyToolbar(mainvbox, toolbardef);
 
-  toolbar = gtk_toolbar_new(GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
   gtk_box_pack_start(GTK_BOX(mainvbox), toolbar, FALSE, FALSE, 5);
   gtk_widget_show(toolbar);
-  gtk_widget_realize(toolbar);
   
-  i = 0;
-  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
-  while (toolbardef[i].label != NULL) {
-    if (strlen(toolbardef[i].label) == 0) {
-      gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
-    } else {
-      GtkWidget *gtkpix = NULL, *gtkpix_grey = NULL;
-      GtkWidget *tb_item;
-      
-      if (toolbardef[i].pixmap != NULL) {
-	GdkBitmap *mask;
-	GdkPixmap *gdkpix, *gdkpix_grey;
-	GtkStyle *style;
-	
-        style = gtk_widget_get_style(toolbar);
-        gdkpix = gdk_pixmap_create_from_xpm_d(toolbar->window,
-                                              &mask,
-                                              &style->bg[GTK_STATE_NORMAL],
-                                              toolbardef[i].pixmap);
-        gtkpix = gtk_pixmap_new(gdkpix, mask);
-        gtk_widget_show(gtkpix);
-
-	/* Create the Grey pixmap */
-	gdkpix_grey =
-	  gdk_pixmap_create_from_xpm_d(toolbar->window,
-				       &mask,
-				       &style->bg[GTK_STATE_NORMAL],
-				       toolbardef[i].grey_pixmap);
-        gtkpix_grey = gtk_pixmap_new(gdkpix_grey, mask);
-        gtk_widget_show(gtkpix_grey);
-      }
-      
-      tb_item = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
-					toolbardef[i].label,
-					toolbardef[i].tooltip,
-					"Private",
-					gtkpix,
-					toolbardef[i].callback,
-					toolbardef[i].data);
-
-      gtk_object_set_data(GTK_OBJECT(tb_item),
-			  NORM_KEY,
-			  gtkpix);
-      gtk_object_set_data(GTK_OBJECT(tb_item),
-			  GREY_KEY,
-			  gtkpix_grey);
-
-      *toolbardef[i].store_to = tb_item;
-    }
-    i++;
-  }
+  SetMyToolbarConfig(tbc);
+  UpdateButtonSensitivity();
 }
 
 static void open_cancel_callback(GtkWidget *widget, gpointer data) {
   GtkWidget *dialog = GTK_WIDGET(data);
+
+  if (open_dialog_status == DIALOG_LIST) {
+    /* Hide list */
+    gtk_notebook_set_page(GTK_NOTEBOOK(open_dialog_notebook),
+			  DIALOG_FIELDS);
+
+    gtk_list_clear_items(GTK_LIST(open_dialog_list), 0, -1);
+  }
   
   gtk_grab_remove(dialog);
   gtk_widget_hide(dialog);
+
+  open_dialog_status = DIALOG_FIELDS;
+}
+
+static void convert_glist(GList *blist) {
+  while (blist) {
+    GtkWidget *list_item;
+    
+    list_item = gtk_list_item_new_with_label((char *) blist->data);
+    gtk_widget_show(list_item);
+    
+    g_free(blist->data);
+    blist->data = list_item;
+
+    blist = blist->next;
+  }
 }
 
 static void open_ok_callback(GtkWidget *widget, gpointer data) {
   DBConnector **connector_table = (DBConnector **) data;
   GtkWidget *notebook = GTK_WIDGET(connector_table[0]);
+  GList *bases_list;
   int page;
+  int return_status;
   
   /* First, we get the page number */
   page = gtk_notebook_current_page(GTK_NOTEBOOK(notebook));
@@ -536,20 +550,66 @@ static void open_ok_callback(GtkWidget *widget, gpointer data) {
 		  "Internal error : connection should be closed");
     return;
   }
-  conn = connector_table[page]->connect_callback(connector_table[page]->data);
-
-  if (conn != NULL) {
-    char buf[256];
-
-    gtk_grab_remove(open_dialog);
-    gtk_widget_hide(open_dialog);
-
-    TablesNewConnection();
-    QueriesNewConnection();
-    UpdateButtonSensitivity();
+  
+  if (open_dialog_status == DIALOG_FIELDS) {
+    return_status =
+      connector_table[page]->connect_callback(connector_table[page]->data,
+					      &conn,
+					      &bases_list,
+					      NULL);
+  } else {
+    /* In that case, we need to get the base number */
+    GList *sel;
+    GtkLabel *label;
+    char *buf;
+    
+    sel = GTK_LIST(open_dialog_list)->selection;
+    label = GTK_LABEL(gtk_container_children(GTK_CONTAINER(sel->data))->data);
+    gtk_label_get(label, &buf);
+    
+    return_status =
+      connector_table[page]->connect_callback(connector_table[page]->data,
+					      &conn,
+					      &bases_list,
+					      buf);
+  }
+    
+  if (return_status == CONNECT_OK) {
+    if (conn != NULL) {
+      char buf[256];
       
-    sprintf(buf, "GtkSQL: %s", conn->DBget_db_name(conn));
-    gtk_window_set_title(GTK_WINDOW(window), buf);
+      gtk_grab_remove(open_dialog);
+      gtk_widget_hide(open_dialog);
+	
+      TablesNewConnection();
+      QueriesNewConnection();
+      UpdateButtonSensitivity();
+      
+      sprintf(buf, "GtkSQL: %s", conn->DBget_db_name(conn));
+      gtk_window_set_title(GTK_WINDOW(window), buf);
+      
+      open_dialog_status = DIALOG_FIELDS;
+      gtk_notebook_set_page(GTK_NOTEBOOK(open_dialog_notebook),
+			    DIALOG_FIELDS);
+
+    }
+  } else if (return_status == CONNECT_LIST) {
+    /* Display list */
+    open_dialog_status = DIALOG_LIST;
+    
+    /* Put elements in the list */
+    convert_glist(bases_list);
+    gtk_list_clear_items(GTK_LIST(open_dialog_list),
+			 0, -1);
+    gtk_list_append_items(GTK_LIST(open_dialog_list),
+			  bases_list);
+    
+    gtk_notebook_set_page(GTK_NOTEBOOK(open_dialog_notebook),
+			  DIALOG_LIST);
+
+    /* I do not understand why I can't free this list here :/
+       
+       g_list_free(bases_list); */
   }
 }
 
@@ -568,77 +628,120 @@ static void connect_to_base(GtkWidget *widget, gpointer data) {
       display_error("Disconnect", buf);
     }
   } else
-    {
-      display_error("Internal error",
-		    "Internal error : open dialog uninitialized");
-      fprintf(stderr, "Internal error, cannot open dialog\n");
-    }
+    display_error("Internal error",
+		  "Internal error : open dialog uninitialized");
 }
 
 static void CreateOpenDialog() {
-  int nb_db, i;
-  DBConnector **connector_table;
-  GtkWidget *ok_button, *cancel_button, *notebook;
+  int nb_db;
   
   /* Count the number of databases supported */
   nb_db = 0;
   while (register_DB[nb_db] != NULL)
     nb_db++;
 
-  /* Create the result table */
-  connector_table = (DBConnector **) malloc((nb_db + 1) *
-					    sizeof(DBConnector *));
-
-  /* Creates the top-level dialog */
-  open_dialog = gtk_dialog_new();
-  gtk_window_set_title(GTK_WINDOW(open_dialog), "Database connection");
-
-  /* Buttons... */
-  ok_button = gtk_button_new_with_label("Ok");
-  gtk_signal_connect(GTK_OBJECT(ok_button), "clicked",
-		     GTK_SIGNAL_FUNC(open_ok_callback), connector_table);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(open_dialog)->action_area),
-		     ok_button, FALSE, FALSE, 0);
-  GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
-  gtk_widget_grab_default(ok_button);
-  gtk_widget_show(ok_button);
-  cancel_button = gtk_button_new_with_label("Cancel");
-  gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked",
-		     GTK_SIGNAL_FUNC(open_cancel_callback), open_dialog);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(open_dialog)->action_area),
-		     cancel_button, FALSE, FALSE, 0);
-  gtk_widget_show(cancel_button);
-
-  /* The notebook */
-  notebook = gtk_notebook_new();
-  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(open_dialog)->vbox),
-		     notebook, FALSE, FALSE, 0);
-  gtk_widget_show(notebook);
-  
-  /* Add the pages to the dialog and the callback functions */
-  connector_table[0] = (DBConnector *) notebook;
-  for (i = 0; i < nb_db; i++) {
-    GtkWidget *pane, *label;
+  if (nb_db == 0) {
+    display_error("Compilation error",
+		  "Compilation error : you do not have any database driver "
+		  "installed. \nPlease check your Makefile to see if you "
+		  "have any of the USE_XXX defined ");
+  } else {
+    GtkWidget *ok_button, *cancel_button, *notebook;
+    GtkWidget *list_label, *list_vbox;
+    int i;
     
-    connector_table[i + 1] = register_DB[i](&pane, &label, display_error);
-    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
-			     pane, label);
+    /* Create the result table */
+    connector_table = (DBConnector **) malloc((nb_db + 2) *
+					      sizeof(DBConnector *));
+    
+    /* Creates the top-level dialog */
+    open_dialog = gtk_dialog_new();
+    gtk_window_set_title(GTK_WINDOW(open_dialog), "Database connection");
+    
+    /* Buttons... */
+    ok_button = gtk_button_new_with_label("Ok");
+    gtk_signal_connect(GTK_OBJECT(ok_button), "clicked",
+		       GTK_SIGNAL_FUNC(open_ok_callback), connector_table);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(open_dialog)->action_area),
+		       ok_button, FALSE, FALSE, 0);
+    GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
+    gtk_widget_grab_default(ok_button);
+    gtk_widget_show(ok_button);
+    cancel_button = gtk_button_new_with_label("Cancel");
+    gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked",
+		       GTK_SIGNAL_FUNC(open_cancel_callback), open_dialog);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(open_dialog)->action_area),
+		       cancel_button, FALSE, FALSE, 0);
+    gtk_widget_show(cancel_button);
+
+    /* The List / Fields notebook */
+    open_dialog_notebook = gtk_notebook_new();
+    gtk_notebook_set_show_tabs(GTK_NOTEBOOK(open_dialog_notebook), FALSE);
+    gtk_notebook_set_show_border(GTK_NOTEBOOK(open_dialog_notebook), FALSE); 
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(open_dialog)->vbox),
+		       open_dialog_notebook, FALSE, FALSE, 0);
+    gtk_widget_show(open_dialog_notebook);
+
+    /* The fields notebook */
+    notebook = gtk_notebook_new();
+    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
+    gtk_notebook_append_page(GTK_NOTEBOOK(open_dialog_notebook),
+			     notebook, NULL);
+    gtk_widget_show(notebook);
+    
+    /* Add the pages to the dialog and the callback functions */
+    connector_table[0] = (DBConnector *) notebook;
+    for (i = 0; i < nb_db; i++) {
+      GtkWidget *pane, *label;
+      
+      connector_table[i + 1] = register_DB[i](&pane, &label, display_error,
+					      connector_table,
+					      open_ok_callback);
+      gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
+			       pane, label);
+    }
+    connector_table[i + 1] = NULL;
+
+    /* The list page */
+    list_vbox = gtk_vbox_new(FALSE, 0);
+    gtk_widget_show(list_vbox);
+    
+    list_label = gtk_label_new("Please choose a base : ");
+    gtk_widget_show(list_label);
+    open_dialog_list = gtk_list_new();
+    gtk_widget_show(open_dialog_list);
+
+    gtk_box_pack_start(GTK_BOX(list_vbox), list_label,
+		       FALSE, FALSE, 0);
+    gtk_box_pack_end(GTK_BOX(list_vbox), open_dialog_list,
+		     TRUE, TRUE, 0);
+
+    gtk_notebook_append_page(GTK_NOTEBOOK(open_dialog_notebook),
+			     list_vbox, NULL);
+
+    gtk_notebook_set_page(GTK_NOTEBOOK(open_dialog_notebook), 0);
+    
+    open_dialog_status = DIALOG_FIELDS;
+    
+    UpdateButtonSensitivity();
   }
 }
 
-int main (int argc, char *argv[])
+int main(int argc, char *argv[])
 {
   GtkWidget *query, *mainvbox, *mainvpane, *menubar, *statusbar;
   GtkWidget *tables, *hpane, *result;
   
+  /* Opens the saved configuration */
+  GetConfig();
+  
   /* Window / GTK initialisation */
   gtk_init(&argc, &argv);
   window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  gtk_widget_set_usize(window, 700, 400);
   gtk_window_set_title(GTK_WINDOW(window), APPNAME);
   gtk_signal_connect(GTK_OBJECT(window), "destroy",
 		     GTK_SIGNAL_FUNC(destroy), NULL);
+  gtk_widget_set_usize(window, 720, 400);
   
   /* Top level layout */
   mainvbox = gtk_vbox_new(FALSE, 0);
@@ -676,15 +779,15 @@ int main (int argc, char *argv[])
   gtk_widget_show(menubar);
   gtk_widget_show(mainvbox);
   gtk_widget_show(window);
-
+  
   /* Various other initialisations */
   UpdateButtonSensitivity();
 
   /* Creates the "open database" dialog */
   CreateOpenDialog();
-  
+
   /* Main loop */
-  gtk_main ();
+  gtk_main();
   
   return 0;
 }
