@@ -23,6 +23,8 @@
 
 DBConnector *register_PostgreSQL(GtkWidget **notebook_pane,
 				 GtkWidget **notebook_label,
-				 void (*error_dialog)(char *, char *)) ;
+				 void (*error_dialog)(char *, char *),
+				 DBConnector **ret_param,
+				 void (*ok_callback)(GtkWidget *, gpointer)) ;
 
 #endif  __POSTGRES_H__
