@@ -50,6 +50,7 @@ typedef struct _key_style {
 
 static GtkWidget *query_notebook;
 static GtkWidget *result_notebook;
+static guint switch_handler;
 
 static int query_status = 0;
 static int nb_queries = 0;
@@ -68,6 +69,9 @@ typedef struct _change_name_dialog {
 int NbQueries() {
   return nb_queries;
 }
+void InitQueries() {
+  last_nb = 0;
+}
 
 /* Adds a keyword in the list in alphabetical order */
 void AddKeyword(DBConnection *conn, char *key, int type) {
@@ -212,8 +216,8 @@ static gint key_press_event(GtkWidget *widget, GdkEventKey *event,
       gtk_notebook_prev_page(GTK_NOTEBOOK(query_notebook));
 
       curpage = gtk_notebook_current_page(GTK_NOTEBOOK(query_notebook));
-      gtk_widget_grab_focus(gtk_notebook_get_page(GTK_NOTEBOOK(query_notebook),
-						  curpage));
+      gtk_widget_grab_focus(gtk_notebook_get_nth_page(GTK_NOTEBOOK(query_notebook),
+						      curpage));
 
       return (TRUE);
     }
@@ -223,8 +227,8 @@ static gint key_press_event(GtkWidget *widget, GdkEventKey *event,
       gtk_notebook_next_page(GTK_NOTEBOOK(query_notebook));
       
       curpage = gtk_notebook_current_page(GTK_NOTEBOOK(query_notebook));
-      gtk_widget_grab_focus(gtk_notebook_get_page(GTK_NOTEBOOK(query_notebook),
-						  curpage));
+      gtk_widget_grab_focus(gtk_notebook_get_nth_page(GTK_NOTEBOOK(query_notebook),
+						      curpage));
 
       return (TRUE);
     }
@@ -355,6 +359,7 @@ void DisplayResults(void *result, int why, int position) {
     gtk_notebook_set_page(GTK_NOTEBOOK(result_notebook), position);
   } else { /* NEW and ERROR */
     GtkWidget *result_widget;
+    GtkWidget *result_clist;
     
     if (why == QUERY_ERROR) {
       if (result == NULL)
@@ -367,9 +372,6 @@ void DisplayResults(void *result, int why, int position) {
       char **col_titles;
       GdkFont *font;
       
-      if (result == NULL)
-	return;
-      
       status_push("Displaying query results...");
       
       if (conn->DBget_query_status(conn, result) == DB_QUERY_RECORDS_OK) {
@@ -380,18 +382,23 @@ void DisplayResults(void *result, int why, int position) {
 	col_titles = (char **) g_malloc(nb_cols * sizeof(char *));
 	for (i = 0; i < nb_cols; i++)
 	  col_titles[i] = conn->DBget_field_name(conn, result, i);
+
+	result_widget = gtk_scrolled_window_new(NULL, NULL);
+	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(result_widget),
+				       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	
-	result_widget = gtk_clist_new_with_titles(nb_cols, col_titles);
-	gtk_clist_set_policy(GTK_CLIST(result_widget),
-			     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	font = gtk_widget_get_style(result_widget)->font;
+	result_clist = gtk_clist_new_with_titles(nb_cols, col_titles);
+	gtk_widget_show(result_clist);
+	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(result_widget), result_clist);
+
+	font = gtk_widget_get_style(result_clist)->font;
 	maxwidth = MAXCOLWID * gdk_char_width(font, 'X');
 	for (i = 0; i < nb_cols; i++)
-	  gtk_clist_set_column_width(GTK_CLIST(result_widget), i,
+	  gtk_clist_set_column_width(GTK_CLIST(result_clist), i,
 				     gdk_string_width(font, col_titles[i]));
 	
 	/* Fill the CList */
-	gtk_clist_freeze(GTK_CLIST(result_widget));
+	gtk_clist_freeze(GTK_CLIST(result_clist));
 	for (i = 0; i < nb_tuples; i++) {
 	  for (j = 0; j < nb_cols; j++) {
 	    int width;
@@ -399,25 +406,25 @@ void DisplayResults(void *result, int why, int position) {
 	    col_titles[j] = conn->DBget_field_value(conn, result, i, j);
 	    width = gdk_string_width(font, col_titles[j]);
 	    
-	    if (width > GTK_CLIST(result_widget)->column[j].width) {
+	    if (width > GTK_CLIST(result_clist)->column[j].width) {
 	      if (width > maxwidth)
-		gtk_clist_set_column_width(GTK_CLIST(result_widget),
+		gtk_clist_set_column_width(GTK_CLIST(result_clist),
 					   j, maxwidth);
 	      else
-		gtk_clist_set_column_width(GTK_CLIST(result_widget),
+		gtk_clist_set_column_width(GTK_CLIST(result_clist),
 					   j, width);   
 	    }
 	  }
-	  gtk_clist_append(GTK_CLIST(result_widget), col_titles);
+	  gtk_clist_append(GTK_CLIST(result_clist), col_titles);
 	}
-	gtk_clist_thaw(GTK_CLIST(result_widget));
+	gtk_clist_thaw(GTK_CLIST(result_clist));
 	g_free(col_titles);
       } else if (conn->DBget_query_status(conn, result) == DB_QUERY_COMMAND_OK)
 	result_widget = gtk_label_new("Command ok.");
       else if (conn->DBget_query_status(conn, result) == DB_QUERY_EMPTY)
 	result_widget = gtk_label_new("Empty query.");
       else
-	result_widget = gtk_label_new("Unexpected return result (error).");
+	result_widget = gtk_label_new("Error in query.");
 
       status_pop();
     }
@@ -432,7 +439,7 @@ void DisplayResults(void *result, int why, int position) {
 GtkWidget *GetCurrentQuery() {
   int cur_page = gtk_notebook_current_page(GTK_NOTEBOOK(query_notebook));
   
-  return gtk_notebook_get_page(GTK_NOTEBOOK(query_notebook), cur_page);
+  return gtk_notebook_get_nth_page(GTK_NOTEBOOK(query_notebook), cur_page);
 }
 
 void *GetCurrentQueryResult() {
@@ -450,7 +457,9 @@ char *GetCurrentQueryName() {
   int position = gtk_notebook_current_page(GTK_NOTEBOOK(query_notebook));
   char *old_name;
   
-  label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(query_notebook), position);
+  label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(query_notebook),
+				     gtk_notebook_get_nth_page(GTK_NOTEBOOK(query_notebook),
+							       position));
   gtk_label_get(GTK_LABEL(label), &old_name);
 
   return old_name;
@@ -473,10 +482,9 @@ void SendQuery() {
   gtk_object_set_data(GTK_OBJECT(current_query),
 		      RESULT_KEY,
 		      result);
-  if ((result == NULL) ||
-      ((conn->DBget_query_status(conn, result) != DB_QUERY_RECORDS_OK) &&
-       (conn->DBget_query_status(conn, result) != DB_QUERY_COMMAND_OK) &&
-       (conn->DBget_query_status(conn, result) != DB_QUERY_EMPTY))) {
+  if ((conn->DBget_query_status(conn, result) != DB_QUERY_RECORDS_OK) &&
+      (conn->DBget_query_status(conn, result) != DB_QUERY_COMMAND_OK) &&
+      (conn->DBget_query_status(conn, result) != DB_QUERY_EMPTY)) {
     GtkWidget *dialog;
     char buf[256];
 
@@ -501,8 +509,8 @@ void SendQuery() {
     DisplayResults(result, QUERY_NEW,
 		   gtk_notebook_current_page(GTK_NOTEBOOK(query_notebook)));
     sprintf(status, "Query results : %d field%sand %d record%s",
-	    nbf, (nbf > 1 ? "s ": " "),
-	    nbr, (nbr > 1 ? "s": ""));
+	    nbf, (nbf != 1 ? "s ": " "),
+	    nbr, (nbr != 1 ? "s": ""));
   }
 
   g_free(cquery);
@@ -535,8 +543,6 @@ void AddQuery() {
 		     GTK_SIGNAL_FUNC(changed_event), current_query);
   gtk_signal_connect(GTK_OBJECT(current_query), "key_press_event",
 		     GTK_SIGNAL_FUNC(key_press_event), current_query);
-  /* HACK !! */
-  GTK_TEXT(current_query)->has_cursor = TRUE;
   gtk_widget_show(current_query);
   gtk_object_set_data(GTK_OBJECT(current_query),
 		      RESULT_KEY,
@@ -576,7 +582,7 @@ static int DeleteQueryAtPosition(int position) {
   void *result;
   
   current_query =
-    gtk_notebook_get_page(GTK_NOTEBOOK(query_notebook), position);
+    gtk_notebook_get_nth_page(GTK_NOTEBOOK(query_notebook), position);
   result = (void *) gtk_object_get_data(GTK_OBJECT(current_query),
 					RESULT_KEY);
 
@@ -587,11 +593,11 @@ static int DeleteQueryAtPosition(int position) {
     conn->DBclear_query(conn, result);
 
   if (position > 0)
-    gtk_widget_grab_focus(gtk_notebook_get_page(GTK_NOTEBOOK(query_notebook),
-						position - 1));
+    gtk_widget_grab_focus(gtk_notebook_get_nth_page(GTK_NOTEBOOK(query_notebook),
+						    position - 1));
   else if (nb_queries > 1)
-    gtk_widget_grab_focus(gtk_notebook_get_page(GTK_NOTEBOOK(query_notebook),
-						position + 1));
+    gtk_widget_grab_focus(gtk_notebook_get_nth_page(GTK_NOTEBOOK(query_notebook),
+						    position + 1));
   
   gtk_notebook_remove_page(GTK_NOTEBOOK(query_notebook), position);
   gtk_notebook_remove_page(GTK_NOTEBOOK(result_notebook), position);
@@ -625,15 +631,11 @@ static void change_name_ok_callback(GtkWidget *widget, gpointer data) {
 
   if (newname != NULL) {
     int position =  gtk_notebook_current_page(GTK_NOTEBOOK(query_notebook));
-    GtkWidget *label;
+    GtkWidget *page = gtk_notebook_get_nth_page(GTK_NOTEBOOK(query_notebook),
+						position);
     
-    label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(query_notebook),
-				       position);
-    gtk_label_set(GTK_LABEL(label), newname);
-    
-    label = gtk_notebook_get_menu_label(GTK_NOTEBOOK(query_notebook),
-					position);
-    gtk_label_set(GTK_LABEL(label), newname);
+    gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(query_notebook), page, newname);
+    gtk_notebook_set_menu_label_text(GTK_NOTEBOOK(query_notebook), page, newname);
 
     gtk_widget_destroy(cd->dialog);
   } else {
@@ -672,7 +674,9 @@ void RenameQuery() {
   gtk_widget_show(label);
 
   name = gtk_entry_new();
-  label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(query_notebook), position);
+  label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(query_notebook),
+				     gtk_notebook_get_nth_page(GTK_NOTEBOOK(query_notebook),
+							       position));
   gtk_label_get(GTK_LABEL(label), &old_name);
   gtk_entry_set_text(GTK_ENTRY(name), old_name);
   gtk_box_pack_start(GTK_BOX(hbox),
@@ -790,19 +794,6 @@ void LoadQueryDialog(void) {
   gtk_widget_show(file_dialog);
 }
 
-void QueriesNewConnection() {
-  AddQuery();
-}
-
-void QueriesConnectionClosed() {
-  int i, nb = nb_queries;
-  
-  for (i = 0; i < nb; i++)
-    DeleteQueryAtPosition(0);
-  
-  nb_queries = 0;
-}
-  
 static void notebook_selection_callback(GtkWidget *widget, gpointer data) {
   if (data != NULL) {
     GtkWidget *current_query = ((GtkNotebookPage *) data)->child; 
@@ -823,6 +814,28 @@ static void notebook_selection_callback(GtkWidget *widget, gpointer data) {
   }
 }
 
+void QueriesNewConnection() {
+  AddQuery();
+
+  /* Used to prevent bug in query deletions */
+  switch_handler =
+    gtk_signal_connect(GTK_OBJECT(query_notebook), "switch_page",
+		       GTK_SIGNAL_FUNC(notebook_selection_callback), NULL);
+}
+
+void QueriesConnectionClosed() {
+  int i, nb = nb_queries;
+
+  /* Used to prevent bugs... */
+  gtk_signal_disconnect(GTK_OBJECT(query_notebook),
+			switch_handler);
+  
+  for (i = 0; i < nb; i++)
+    DeleteQueryAtPosition(0);
+  
+  nb_queries = 0;
+}
+
 GtkWidget *InitQueryPane()
 {
   /* The query notebook */
@@ -830,8 +843,6 @@ GtkWidget *InitQueryPane()
   gtk_notebook_set_tab_pos(GTK_NOTEBOOK(query_notebook), GTK_POS_TOP);
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(query_notebook), TRUE);
   gtk_notebook_popup_enable(GTK_NOTEBOOK(query_notebook));
-  gtk_signal_connect(GTK_OBJECT(query_notebook), "switch_page",
-		     GTK_SIGNAL_FUNC(notebook_selection_callback), NULL);
   gtk_widget_set_usize(query_notebook, 250, 130);
   gtk_widget_show(query_notebook);
 
